package it.andynaz.utils;

import static java.lang.System.out;


/**
 * <p>Metodi utili generici.</p>
 * 
 * @author andynaz
 * @version 0.4 - 2012/11/11
 */
public abstract class Utils{
	// nasconde il costruttore
	private Utils(){}

	/**
	 * <p>Controlla se una stringa &egrave; vuota.</p>
	 * 
	 * @param s una stringa da conrollare
	 * @return <tt>true</tt> se il riferimento alla stringa &egrave; nullo e se
	 *   la stringa &egrave; vuota (ha lunghezza 0), <tt>false</tt> altrimenti
	 * 
	 * @since 0.1
	 */
	public static boolean isEmpty(String s){
		return ( s==null || s.length()==0 );
	}
	
	
	/**
	 * <p>Scrive sullo standard output il log di un'eccezione o un errore,
	 * preceduti da un messaggio.</p>
	 *
	 * <p>In particolare, stampa nell'ordine
	 * <ol>
	 *   <li>il messaggio personalizzato</li>
	 *   <li>il messaggio dell'eccezione</li>
	 *   <li>lo stack trace dell'eccezione/errore</li>
	 * </ol>
	 * </p>
	 * 
	 * <p><b>Nota:</b> ogni elemento viene stampato solo se <i>significativo</i>
	 * (ovvero se non nullo e non vuoto).</p>
	 *
	 * @param message messaggio personalizzato
	 * @param t errore o eccezione da loggare
	 * 
	 * @since 0.1
	 */
	public static void printException(String message, Throwable t){
		if( !isEmpty(message) ) out.println(message);
		if( !isEmpty(t.getMessage()) ) out.println(t.getMessage());
		if( t.getStackTrace()!=null ) t.printStackTrace();
	}
	
	
	/**
	 * Trasforma una stringa in un intero.
	 * 
	 * Nel caso in cui non sia possibile trasformare la stringa in un intero,
	 * verr&agrave; ritornato il valore <tt>null</tt>.
	 * 
	 * <p>Nota: il metodo non lancia nessuna eccezione di tipo
	 * {@link NumberFormatException}.</p>
	 * 
	 * @param s stringa da parsare
	 * @return l'intero corrispondete alla stringa <tt>s</tt>, o <tt>null</tt> se
	 *   la stringa non rappresenta un intero
	 * 
	 * @since 0.2
	 */
	public static Integer parseInt(String s){
		Integer i = null;
		if( !isEmpty(s) ){
			try{
				i = new Integer(s);
			} catch(NumberFormatException e){}
		}
		return i;
	}
	
	
	/**
	 * Trasforma una stringa in un intero di tipo <tt>Long</tt>.
	 * 
	 * Nel caso in cui non sia possibile trasformare la stringa in un intero,
	 * verr&agrave; ritornato il valore <tt>null</tt>.
	 * 
	 * <p>Nota: il metodo non lancia nessuna eccezione di tipo
	 * {@link java.lang.NumberFormatException NumberFormatException}.</p>
	 * 
	 * @param s stringa da parsare
	 * @return l'intero corrispondete alla stringa <tt>s</tt>, o <tt>null</tt> se
	 *   la stringa non rappresenta un intero
	 * 
	 * @since 0.2
	 */
	public static Long parseLong(String s){
		Long l = null;
		if( !isEmpty(s) ){
			try{
				l = new Long(s);
			} catch(NumberFormatException e){}
		}
		return l;
	}
	
	
	/**
	 * <p>Fa il padding a sinistra della stringa <code>s</code>, con il pattern
	 * <code>p</code>, fino a raggiungere (almeno) la lunghezza
	 * <code>l</code>.</p>
	 * 
	 * <p><b>Nota:</b> la stringa restituita potrebbe essere pi&ugrave; lunga di
	 * <code>l</code> caratteri.</p>
	 *
	 * @param s stringa di cui fare il padding
	 * @param p pattern con cui fare il padding
	 * @param l lunghezza minima della stringa da restituire
	 * 
	 * @throws NullPointerException se almeno una tra <code>s</code> o
	 * <code>p</code> &egrave; <tt>null</tt>.
	 * @throws IllegalArgumentException se la lunghezza indicata &egrave;
	 * negativa o se il pattern <code>p</code> &egrave; vuoto
	 * 
	 * @since 0.3
	 */
	public static String pad(String s, String p, int l){
		if( s==null ) throw new NullPointerException("stringa s nulla");
		if( p==null ) throw new NullPointerException("pattern p nullo");
		if( p.length()==0 ) throw new IllegalArgumentException("pattern vuoto");
		if( l<0 ) throw new IllegalArgumentException("lunghezza negativa: n="+l);
	
		StringBuilder sb = new StringBuilder(l);
		sb.append(s);
		while( sb.length()<l )
			sb.insert(0, p);
		return sb.toString();
	}
	

	/**
	 * Concatena le stringhe passate come argomento separandole con la stringa
	 * <tt>sep</tt>.
	 * 
	 * Nel caso in cui <tt>sep</tt> sia vuota, concatena le stringhe senza nessun
	 * separatore.
	 * 
	 * @param s stringhe da concatenare
	 * @param separator separatore
	 * 
	 * @throws NullPointerException nel caso in cui <code>s</code> sia nullo
	 * @throws IllegalArgumentException nel caso in cui <code>s</code> abbia
	 * lunghezza 0
	 * 
	 * @since v0.4
	 */
	public static String concat(String[] s, String separator){
		if( s==null ) throw new NullPointerException("array di stringhe nullo");
		if( s.length==0 ) throw new IllegalArgumentException("array di stringhe vuoto");
		
		String sep = isEmpty(separator) ? "" : separator;
		
		StringBuilder sb = new StringBuilder(s.length*10);
		for( int i=0; i<s.length; i++ ){
			if( i>0 ) sb.append(sep);
			sb.append(s[i]);
		}
		return sb.toString();
	}
	
}
